/* p12.c */
/*
 * Copyright (c) 2004-2014 National Institute of Informatics in Japan,
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must
 * include this notice in the file.
 */
/*
 * Copyright (C) 1998-2004
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <aicrypto/ok_asn1.h>

int print_pkcs12(char *fname){
  unsigned char *der,*ctx,*ctx2,*cp,*t;
  unsigned char *mac,*p7c,*p8c;
  int	i,j;

  if((der = ASN1_read_der(fname))==NULL)
      return 0;

  /* PKCS#12 version check */
  cp = ASN1_next(der);
  if(ASN1_integer(cp,&i)!=3) return 0; /* not PKCS#12 file */

  OK_ASN1_print(der);

  cp = ASN1_next(cp);
  if((ctx=ASN1_get_p7data(cp,&i))==NULL)
      return 0; /* not PKCS#12 file */

  cp = ASN1_skip(cp);

  if(P12_verify_mac("Input PKCS#12 Password: ",cp,ctx)){/* password error */
      printf("MAC Verify Error -- password missing.\n");
      free(der);
      goto done;
  }

  cp = ASN1_find_tag(cp,ASN1_OCTETSTRING);
  ASN1_octetstring(cp,&i,&mac,&j);
  printf("p12mac: ");
  for(i=0;i<j;i++) printf("%.2x ",mac[i]);
  printf(" -- %d\n",j);

/* --- --- --- --- --- */
  OK_ASN1_print(ctx);

  /* microsoft PKCS#12 is KeyBag first and CertBag last.
   * but netscape old type is CertBag first and KeyBag last ;-(
   */
  cp = ASN1_next(ctx);
  t  = ASN1_next(cp);

  if(ASN1_object_2int(t)==OBJ_P7_ENCRYPTED){
      /* this is netscape type */
	  t = ASN1_skip(cp);
      if((ctx2=ASN1_get_p7data(t,&i))==NULL) goto done;

  }else{
      /* this is microsoft type */
      if((ctx2=ASN1_get_p7data(cp,&i))==NULL) goto done;
      cp = ASN1_skip(cp);
  }

  if((p7c=ASN1_get_p7enc(cp,&i))==NULL) goto done;

  printf("-- decrypted PKCS7encrypt --\n");
  OK_ASN1_print(p7c);
  printf("-- decrypted PKCS7encrypt --\n");

/* --- --- --- --- --- */
  OK_ASN1_print(ctx2);

  cp = ASN1_find_tag(ctx2,ASN1_OBJECT_IDENTIFIER);
  if(ASN1_object_2int(cp)!=OBJ_P12v1Bag_PKCS8) goto done;

  cp = ASN1_step(cp,2);
  if((p8c=ASN1_p8_decrypted(cp,&j))==NULL) goto done;

  printf("-- decrypted Bag PKCS8 --\n");
  OK_ASN1_print(p8c);
  printf("-- decrypted Bag PKCS8 --\n");

done:
  return 1;
}

